@@ -19,6 +19,7 @@ Ollama's simple API.
 """
 
 import logging
+import os
 from dataclasses import dataclass
 from typing import Any, Dict, List, Optional
 
@@ -26,6 +27,7 @@ from .llama_cpp_coordinator import LlamaCppCoordinator, RPCBackend
 from .ollama_gguf_resolver import OllamaGGUFResolver
 from .pool import OllamaPool
 from .rpc_registry import RPCBackendRegistry
+from .routing_logger import get_routing_logger
 
 logger = logging.getLogger(__name__)
 
@@ -97,6 +99,7 @@ class HybridRouter:
         auto_setup_rpc: bool = False,
         num_rpc_backends: int = 1,
         auto_fallback: bool = True,
+        debug_coordinator_recovery: bool = False,
     ):
         """
         Initialize hybrid router with automatic GGUF resolution from Ollama.
@@ -112,9 +115,16 @@ class HybridRouter:
             auto_setup_rpc: Auto-setup RPC backends if none found (requires llama.cpp)
             num_rpc_backends: Number of RPC backends to start if auto-setup is enabled
             auto_fallback: Automatically fallback to RPC if Ollama fails (default: True)
+            debug_coordinator_recovery: Enable verbose logging for coordinator crash recovery (default: False)
+                                       Can also be enabled via SOLLOL_DEBUG_COORDINATOR_RECOVERY=true env var
         """
         self.ollama_pool = ollama_pool
         self.auto_fallback = auto_fallback
+        # Allow environment variable to override debug setting
+        self.debug_coordinator_recovery = (
+            debug_coordinator_recovery or
+            os.getenv("SOLLOL_DEBUG_COORDINATOR_RECOVERY", "").lower() in ("true", "1", "yes", "on")
+        )
 
         # Cache for routing decisions (model -> use_rpc)
         self.routing_cache = {}
@@ -168,6 +178,9 @@ class HybridRouter:
         # GGUF resolver for extracting models from Ollama storage
         self.gguf_resolver = OllamaGGUFResolver()
 
+        # Initialize routing event logger
+        self.routing_logger = get_routing_logger()
+
         # Log initialization status
         rpc_info = ""
         if self.enable_distributed:
@@ -180,6 +193,19 @@ class HybridRouter:
             f"{rpc_info}"
         )
 
+        # Auto-start observability dashboard (configurable via ENV)
+        self.dashboard = None
+        self.dashboard_enabled = os.getenv("SOLLOL_DASHBOARD", "true").lower() in ("true", "1", "yes", "on")
+        self.dashboard_port = int(os.getenv("SOLLOL_DASHBOARD_PORT", "8080"))
+        self.dashboard_enable_dask = os.getenv("SOLLOL_DASHBOARD_DASK", "true").lower() in ("true", "1", "yes", "on")
+
+        if self.dashboard_enabled:
+            self._start_dashboard()
+        else:
+            msg = "📊 Dashboard disabled via SOLLOL_DASHBOARD=false"
+            logger.info(msg)
+            print(msg)
+
     async def _ensure_coordinator_for_model(self, model: str):
         """
         Ensure coordinator is started with the correct model.
@@ -220,6 +246,12 @@ class HybridRouter:
             logger.info(
                 f"Stopping coordinator (switching from {self.coordinator_model} to {model})"
             )
+            self.routing_logger.log_model_switch(
+                from_model=self.coordinator_model,
+                to_model=model,
+                backend="llamacpp"
+            )
+            self.routing_logger.log_coordinator_stop(model=self.coordinator_model)
             await self.coordinator.stop()
             self.coordinator = None
 
@@ -249,6 +281,12 @@ class HybridRouter:
                     f"✅ Coordinator started with {len(backends)} RPC backends "
                     f"on {self.coordinator_host}:{self.coordinator_port}"
                 )
+                self.routing_logger.log_coordinator_start(
+                    model=model,
+                    rpc_backends=len(backends),
+                    coordinator_host=self.coordinator_host,
+                    coordinator_port=self.coordinator_port
+                )
             except Exception as e:
                 # Startup failed - clean up the failed coordinator
                 logger.error(f"Coordinator startup failed: {e}")
@@ -371,9 +409,21 @@ class HybridRouter:
             use_rpc = self.routing_cache[model]
             if use_rpc:
                 logger.info(f"🔗 Routing '{model}' to RPC (cached decision)")
+                self.routing_logger.log_route_decision(
+                    model=model,
+                    backend="rpc",
+                    reason="cached_routing_decision",
+                    cached=True
+                )
                 return await self._route_to_llamacpp(model, messages, **kwargs)
             else:
                 logger.info(f"📡 Routing '{model}' to Ollama (cached decision)")
+                self.routing_logger.log_route_decision(
+                    model=model,
+                    backend="ollama",
+                    reason="cached_routing_decision",
+                    cached=True
+                )
                 return await self._route_to_ollama(model, messages, **kwargs)
 
         # No cached decision - check resources proactively
@@ -385,6 +435,13 @@ class HybridRouter:
                 # Resources sufficient - try Ollama
                 try:
                     logger.info(f"📡 Routing '{model}' to Ollama pool (sufficient resources)")
+                    profile = self._get_model_profile(model)
+                    self.routing_logger.log_route_decision(
+                        model=model,
+                        backend="ollama",
+                        reason=f"sufficient_resources (estimated {profile.estimated_memory_gb:.1f}GB)",
+                        parameter_count=profile.parameter_count
+                    )
                     result = await self._route_to_ollama(model, messages, **kwargs)
                     # Success! Cache this decision
                     self.routing_cache[model] = False
@@ -396,6 +453,12 @@ class HybridRouter:
                             f"⚠️  Ollama failed unexpectedly for '{model}': {str(e)[:100]}"
                         )
                         logger.info(f"🔗 Falling back to RPC model sharding...")
+                        self.routing_logger.log_fallback(
+                            model=model,
+                            from_backend="ollama",
+                            to_backend="rpc",
+                            reason=f"ollama_error: {str(e)[:100]}"
+                        )
                         result = await self._route_to_llamacpp(model, messages, **kwargs)
                         self.routing_cache[model] = True
                         return result
@@ -405,6 +468,13 @@ class HybridRouter:
                 # Insufficient resources - use RPC if available
                 if self.enable_distributed:
                     logger.info(f"🔗 Routing '{model}' to RPC (insufficient Ollama resources)")
+                    profile = self._get_model_profile(model)
+                    self.routing_logger.log_route_decision(
+                        model=model,
+                        backend="rpc",
+                        reason=f"insufficient_ollama_resources (requires {profile.estimated_memory_gb:.1f}GB)",
+                        parameter_count=profile.parameter_count
+                    )
                     result = await self._route_to_llamacpp(model, messages, **kwargs)
                     self.routing_cache[model] = True
                     return result
@@ -420,6 +490,11 @@ class HybridRouter:
         # No Ollama pool available - use RPC directly
         if self.enable_distributed:
             logger.info(f"🔗 Routing '{model}' to RPC (no Ollama pool)")
+            self.routing_logger.log_route_decision(
+                model=model,
+                backend="rpc",
+                reason="no_ollama_pool_available"
+            )
             result = await self._route_to_llamacpp(model, messages, **kwargs)
             self.routing_cache[model] = True
             return result
@@ -455,16 +530,67 @@ class HybridRouter:
         2. Starts the coordinator with the correct model
         3. Distributes model layers across RPC backends
         4. Makes the inference request
+        5. Handles coordinator crashes with automatic retry
         """
         # Ensure coordinator is started with correct model (auto-resolves GGUF!)
         await self._ensure_coordinator_for_model(model)
 
-        # Use the coordinator's chat method (which uses /v1/chat/completions endpoint)
-        result = await self.coordinator.chat(
-            messages=messages,
-            max_tokens=kwargs.get("max_tokens", 512),
-            temperature=kwargs.get("temperature", 0.7),
-        )
+        # Use the coordinator's chat method with crash recovery
+        try:
+            result = await self.coordinator.chat(
+                messages=messages,
+                max_tokens=kwargs.get("max_tokens", 512),
+                temperature=kwargs.get("temperature", 0.7),
+            )
+        except Exception as e:
+            # Coordinator crashed or failed - attempt recovery once
+            logger.warning(f"⚠️  Coordinator failed for '{model}': {str(e)[:100]}")
+
+            if self.debug_coordinator_recovery:
+                logger.info("🔄 Attempting coordinator recovery (restarting)...")
+            else:
+                logger.debug("Attempting coordinator recovery (restarting)...")
+
+            try:
+                # Stop and clear the failed coordinator
+                if self.coordinator:
+                    if self.debug_coordinator_recovery:
+                        logger.debug("Stopping failed coordinator...")
+                    try:
+                        await self.coordinator.stop()
+                    except:
+                        pass  # Ignore errors stopping crashed coordinator
+                    self.coordinator = None
+                    self.coordinator_model = None
+
+                # Restart coordinator with fresh instance
+                if self.debug_coordinator_recovery:
+                    logger.debug(f"Restarting coordinator for model '{model}'...")
+                await self._ensure_coordinator_for_model(model)
+
+                # Retry the request once
+                if self.debug_coordinator_recovery:
+                    logger.info("✅ Coordinator restarted, retrying request...")
+                else:
+                    logger.debug("Coordinator restarted, retrying request...")
+
+                result = await self.coordinator.chat(
+                    messages=messages,
+                    max_tokens=kwargs.get("max_tokens", 512),
+                    temperature=kwargs.get("temperature", 0.7),
+                )
+
+                if self.debug_coordinator_recovery:
+                    logger.info("✅ Request succeeded after coordinator restart")
+                else:
+                    logger.debug("Request succeeded after coordinator restart")
+
+            except Exception as retry_error:
+                # Recovery failed - log and re-raise
+                logger.error(f"❌ Coordinator recovery failed: {retry_error}")
+                raise RuntimeError(
+                    f"Coordinator failed and recovery attempt unsuccessful: {retry_error}"
+                ) from retry_error
 
         # Convert to Ollama-style response
         return self._convert_llamacpp_to_ollama(result, model)
@@ -559,3 +685,66 @@ class HybridRouter:
             stats["model_sharding"]["rpc_backends"] = len(self.coordinator.rpc_backends)
 
         return stats
+
+    def _start_dashboard(self):
+        """
+        Start the unified observability dashboard in a background thread.
+
+        The dashboard provides real-time monitoring via WebSockets:
+        - System metrics (hosts, latency, success rate, GPU memory, Ray workers)
+        - Real-time logs streaming
+        - Ollama server activity monitoring
+        - llama.cpp RPC activity monitoring
+        - Embedded Ray dashboard iframe
+        - Embedded Dask dashboard iframe
+
+        Configured via environment variables:
+        - SOLLOL_DASHBOARD=true|false (default: true)
+        - SOLLOL_DASHBOARD_PORT=8080 (default: 8080)
+        - SOLLOL_DASHBOARD_DASK=true|false (default: true)
+        """
+        try:
+            from .dashboard_launcher import launch_dashboard_subprocess
+            from .dashboard_log_hooks import install_log_hook_main
+
+            # Get Redis URL from environment
+            redis_url = os.getenv("SOLLOL_REDIS_URL", "redis://localhost:6379")
+
+            msg = f"🚀 Launching SOLLOL Dashboard subprocess on port {self.dashboard_port}..."
+            logger.info(msg)
+            print(msg)
+
+            # Launch dashboard as subprocess
+            self.dashboard = launch_dashboard_subprocess(
+                redis_url=redis_url,
+                port=self.dashboard_port,
+                ray_dashboard_port=8265,
+                dask_dashboard_port=8787,
+                enable_dask=self.dashboard_enable_dask,
+                debug=False,
+            )
+
+            # Install log hooks in main process
+            install_log_hook_main(redis_url)
+
+            msg1 = "✅ SOLLOL Dashboard launched successfully"
+            msg2 = f"   📊 Access at http://localhost:{self.dashboard_port}"
+            msg3 = f"   📡 Using Redis at {redis_url}"
+            msg4 = f"   🔧 Disable with: SOLLOL_DASHBOARD=false"
+
+            logger.info(msg1)
+            logger.info(msg2)
+            logger.info(msg3)
+            logger.info(msg4)
+            print(msg1)
+            print(msg2)
+            print(msg3)
+            print(msg4)
+
+        except Exception as e:
+            err_msg = f"⚠️  Failed to start dashboard: {e}"
+            info_msg = "   Dashboard can be started manually: python -m sollol.dashboard_service"
+            logger.warning(err_msg)
+            logger.info(info_msg)
+            print(err_msg)
+            print(info_msg)
