@@ -10,10 +10,13 @@ Features:
 - Network traffic analysis
 - Error tracking and alerting
 - Activity stream for dashboard
+- Configurable sampling to reduce overhead
 """
 
+import json
 import logging
 import queue
+import random
 import threading
 import time
 from collections import deque
@@ -22,6 +25,12 @@ from datetime import datetime
 from enum import Enum
 from typing import Any, Dict, List, Optional
 
+try:
+    import redis
+    REDIS_AVAILABLE = True
+except ImportError:
+    REDIS_AVAILABLE = False
+
 logger = logging.getLogger(__name__)
 
 
@@ -86,20 +95,35 @@ class NetworkObserver:
     Monitors all Ollama and llama.cpp backend actions in real-time.
     """
 
-    def __init__(self, max_events: int = 10000):
+    def __init__(
+        self,
+        max_events: int = 10000,
+        redis_url: str = "redis://localhost:6379",
+        enable_sampling: bool = True,
+        sample_rate: float = 0.1
+    ):
         """
         Initialize network observer.
 
         Args:
             max_events: Maximum events to keep in memory
+            redis_url: Redis connection URL for dashboard pub/sub
+            enable_sampling: Enable sampling to reduce overhead (default: True)
+            sample_rate: Fraction of info events to log (0.0-1.0, default: 0.1 = 10%)
         """
         self.max_events = max_events
         self.events: deque[NetworkEvent] = deque(maxlen=max_events)
         self.event_queue: queue.Queue = queue.Queue()
 
+        # Sampling configuration
+        self.enable_sampling = enable_sampling
+        self.sample_rate = max(0.0, min(1.0, sample_rate))  # Clamp to [0, 1]
+
         # Statistics tracking
         self.stats = {
             "total_events": 0,
+            "sampled_events": 0,  # Events that passed sampling
+            "dropped_events": 0,  # Events dropped by sampling
             "events_by_type": {},
             "events_by_backend": {},
             "errors_by_backend": {},
@@ -113,6 +137,17 @@ class NetworkObserver:
         self._lock = threading.Lock()
         self._running = True
 
+        # Initialize Redis for dashboard pub/sub
+        self.redis_client = None
+        if REDIS_AVAILABLE:
+            try:
+                self.redis_client = redis.from_url(redis_url, decode_responses=True)
+                self.redis_client.ping()
+                logger.info(f"📡 Network Observer connected to Redis at {redis_url}")
+            except Exception as e:
+                logger.warning(f"Failed to connect to Redis: {e} - dashboard activity logs will be unavailable")
+                self.redis_client = None
+
         # Start event processing thread
         self._event_thread = threading.Thread(
             target=self._process_events_loop,
@@ -121,7 +156,8 @@ class NetworkObserver:
         )
         self._event_thread.start()
 
-        logger.info("🔍 Network Observer initialized (event-driven monitoring)")
+        sampling_status = f"sampling={self.sample_rate:.0%}" if self.enable_sampling else "sampling=disabled"
+        logger.info(f"🔍 Network Observer initialized (event-driven monitoring, {sampling_status})")
 
     def log_event(
         self,
@@ -170,14 +206,24 @@ class NetworkObserver:
         logger.debug("Network Observer event processing thread stopped")
 
     def _process_event(self, event: NetworkEvent):
-        """Process a single event."""
+        """Process a single event with sampling."""
         with self._lock:
+            # Update total event count (before sampling)
+            self.stats["total_events"] += 1
+
+            # Apply sampling for info-level events
+            if self.enable_sampling and event.severity == "info":
+                if random.random() > self.sample_rate:
+                    # Drop this event
+                    self.stats["dropped_events"] += 1
+                    return
+
+            # Event passed sampling (or sampling disabled)
+            self.stats["sampled_events"] += 1
+
             # Add to event history
             self.events.append(event)
 
-            # Update statistics
-            self.stats["total_events"] += 1
-
             event_type_key = event.event_type.value
             self.stats["events_by_type"][event_type_key] = \
                 self.stats["events_by_type"].get(event_type_key, 0) + 1
@@ -215,6 +261,9 @@ class NetworkObserver:
             # Update backend metrics
             self._update_backend_metrics(event)
 
+            # Publish to Redis for dashboard activity logs
+            self._publish_to_dashboard(event)
+
             # Log significant events
             if event.severity in ["warning", "error", "critical"]:
                 logger.info(
@@ -254,6 +303,37 @@ class NetworkObserver:
             if metrics["total_requests"] > 0:
                 metrics["avg_latency_ms"] = metrics["total_latency_ms"] / metrics["total_requests"]
 
+    def _publish_to_dashboard(self, event: NetworkEvent):
+        """Publish event to Redis for dashboard activity logs."""
+        if not self.redis_client:
+            return
+
+        try:
+            # Determine the Redis channel based on event type
+            channel = None
+            if event.event_type in [EventType.OLLAMA_REQUEST, EventType.OLLAMA_RESPONSE, EventType.OLLAMA_ERROR]:
+                channel = "sollol:dashboard:ollama:activity"
+            elif event.event_type in [EventType.RPC_REQUEST, EventType.RPC_RESPONSE, EventType.RPC_ERROR]:
+                channel = "sollol:dashboard:rpc:activity"
+
+            if not channel:
+                return
+
+            # Format event for dashboard
+            message = {
+                "timestamp": event.timestamp,
+                "backend": event.backend,
+                "event_type": event.event_type.value,
+                "severity": event.severity,
+                "details": event.details,
+            }
+
+            # Publish to Redis channel
+            self.redis_client.publish(channel, json.dumps(message))
+
+        except Exception as e:
+            logger.debug(f"Failed to publish event to Redis: {e}")
+
     def get_recent_events(
         self,
         limit: int = 100,
@@ -322,10 +402,17 @@ class NetworkObserver:
     def get_stats(self) -> Dict[str, Any]:
         """Get overall observability statistics."""
         with self._lock:
+            total = self.stats["total_events"]
+            sampled = self.stats["sampled_events"]
+            dropped = self.stats["dropped_events"]
+
             return {
                 **self.stats,
                 "backends_tracked": len(self.backend_metrics),
                 "events_in_memory": len(self.events),
+                "sampling_enabled": self.enable_sampling,
+                "sample_rate": self.sample_rate,
+                "sampling_efficiency": (dropped / total) if total > 0 else 0.0,
             }
 
     def shutdown(self):
